import os.path
import platform

# os.path.exists(path)
# os.path.isfile(path)
# os.path.isdir(path)
# os.path.dirname(__file__)

OSName = os.name

# python script name
ScriptName = __file__
Program         = os.path.basename(__file__)

OS         = platform.system()    # macOS: Darwin  Windows: Windows  Ubuntu: Darwin
OSRelease  = platform.release()   # macOS: 18.2.0  Windows: 10
OSVersion  = platform.version()   # macOS: Darwin-18.2.0-x86...  Windows: 10.0.17763
OSPlatform = platform.platform()  # macOS: Darwin-18.2.0-x86_64-i386-64bit  Windows: Windows-10-10.0.17763-SP0

if OS == 'Linux':
    ProgramDir = os.environ.get("ProgramDir")
    BinDir     = os.environ.get("BinRoot")
    if BinDir is None and ProgramDir is not None:
        BinDir = os.path.join(ProgramDir, 'bin')
else:
    ProgramDir = os.environ.get("ProgramDir", None)
    BinDir     = os.environ.get("BinRoot", None)

    if ProgramDir is None or not os.path.isdir(ProgramDir):
        ProgramDir = "d:\\tkProg\\tklib"
    if not os.path.isdir(ProgramDir):
        ProgramDir = "c:\\tkProg\\tklib" 
    if not os.path.isdir(ProgramDir):
        ProgramDir = "d:\\Programs"
    if not os.path.isdir(ProgramDir):
        ProgramDir = "c:\\Programs" 

    if BinDir is None or not os.path.isdir(BinDir):
        BinDir = os.path.join(ProgramDir, 'bin')

DBDir = os.environ.get("TkDatabasesDir", None)
if DBDir is None:
    DBDir = os.path.join(ProgramDir, 'Databases')
if not os.path.isdir(DBDir):
    DBDir = os.path.join(BinDir, 'Databases')

#print("ProgramDir=", ProgramDir)
#print("BinDir=", BinDir)
#print("DBDir=", DBDir)
#exit()

AtomDBDir = os.path.join(DBDir, 'atoms')

RietanDir = os.path.join(ProgramDir, 'XRD', 'RIETAN-FP')

VASPDir       = os.path.join(ProgramDir, "VASP")
VASPPerlDir   = os.path.join(ProgramDir, 'Perl', 'VASP')
VASPPythonDir = os.path.join(ProgramDir, 'python', 'VASP')


def system(cmd):
    os.system(cmd)

def getenv():
    return os.environ

def setenv(varname, value):
    os.environ[varname] = value

def mkdir(path):
    os.mkdi(path)

def rename(oldpath, newpath):
    os.rename(oldpath, newpath)
    
def remove(path):
    os.remove(path)

def creation_date(path):
    """
    Try to get the date that a file was created, falling back to when it was
    last modified if that isn't possible.
    See http://stackoverflow.com/a/39501288/1709587 for explanation.
    """
    if platform.system() == 'Windows':
        return os.path.getctime(path)
    else:
        stat = os.stat(path)
        try:
            return stat.st_birthtime
        except AttributeError:
            # We're probably on Linux. No easy way to get creation dates here,
            # so we'll settle for when its content was last modified.
            return stat.st_mtime
            

"""
#===============================================
# スクリプト大域変数
#===============================================
my $OS = $^O;
my $ProgramOldDir   = "d:\\Programs.old";
$ProgramOldDir = "c:\\Programs.old" if(!-d $ProgramOldDir);
my $BaseDir         = $ENV{TkPerlDir};
   $BaseDir         = Deps::MakePath($ProgramDir, "Perl", 0) if($BaseDir eq '');
my $TkPerlDir       = $BaseDir;
my $BinDir          = Deps::MakePath($ProgramDir, "bin", 0);
my $DBDir           = Deps::MakePath($ProgramDir, "Databases", 0);
if($OS eq 'MSWin32') {
}
elsif($OS eq 'linux') {
	$ProgramDir = Utils::MakePath('/home/share/', 'bin', '/', 0);
	$ProgramDir = Utils::MakePath($ENV{HOME}, 'bin', '/', 0) if(!-d $ProgramDir);
	$BinDir     = $ProgramDir;
	$DBDir      = Deps::MakePath($ProgramDir, "PeriodicTables", 0);
}
#print "DBDir: $DBDir\n";

my $home = ($ENV{HOME})? $ENV{HOME} : '';

my $QEDir   = Deps::MakePath($TkPerlDir, "QE", 0);
my $QEPPDir = Deps::MakePath($QEDir, "pseudo", 0);

my $PHASEDir     = Deps::MakePath($ProgramDir, ["Quantum", "PHASE"], 0);
   $PHASEDir     = "$home/bin/Perl/PHASE" if(!-d $PHASEDir);
my $PHASEPerlDir = Deps::MakePath($TkPerlDir, ["PHASE"], 0);
   $PHASEPerlDir = "$home/bin/Perl/PHASE" if(!-d $PHASEPerlDir);
my $PHASEPotDir  = Deps::MakePath($ProgramOldDir, ["Quantum", "PHASE", "pp"], 0);
   $PHASEPotDir = "$home/bin/Perl/PHASE" if(!-d $PHASEPotDir);

my $SCIGRESSDir = Deps::MakePath($TkPerlDir, ["SCIGRESS"], 0);

my $CRYSTAL06Dir = Deps::MakePath($ProgramDir, "CRYSTAL", 0);
   unless(-d $CRYSTAL06Dir) {
	my $HOME = '';
	$HOME = $ENV{'HOME'} if($ENV{'HOME'});
	$CRYSTAL06Dir = "$HOME/bin/Perl/CRYSTAL";
   }

my $DVXaDir = Deps::MakePath($ProgramDir, ["Quantum", "DVXa97"], 0);
   $DVXaDir = "$home/bin/Perl/DVXa97" if(!-d $DVXaDir);

our $TSPACEDir           = Deps::MakePath($TkPerlDir,  ["TSPACE"], 0);
our $TSPACEDBDir         = Deps::MakePath($ProgramDir, ["Databases"],     0);

#my $RietanDir           = Deps::MakePath($ProgramDir,  ["X線回折", "Rietan2000"],      0);
my $XRDDir              = Deps::MakePath($ProgramDir,  ["XRD"],      0);
my $RietanDir           = Deps::MakePath($XRDDir,      ["RIETAN-FP"],      0);
my $RietanTemplateDir   = Deps::MakePath2($TkPerlDir,  ["Rietan",   "Template"], 0);
my $RietanFPDir         = Deps::MakePath($XRDDir,      ["Rietan-FP"],      0);
my $RietanFPTemplateDir = Deps::MakePath2($TkPerlDir,  ["RietanFP", "Template"], 0);
if($OS eq 'MSWin32') {
}
elsif($OS eq 'linux') {
	$RietanDir   = "$ENV{HOME}/bin/rietan";
	$RietanFPDir = "$ENV{HOME}/bin/RietanFP";
}

my $PeriodicTableDir = Deps::MakePath($ProgramDir, ["Databases"], 0);
#print "DBDir: $DBDir\n";
#print "PeriodicTableDir: $PeriodicTableDir\n";
#print "RietanDir: $RietanDir\n";

my $MXDDir       = Deps::MakePath($ProgramDir, ["MD", "MXD"],  0);
my $MXDExecDir   = Deps::MakePath($MXDDir,     "exec",         0);
my $MXDOrtoDir   = Deps::MakePath($MXDDir,     "MXDOrto",      0);
my $MXDInputPath = Deps::MakePath($MXDExecDir, "MXDInput.exe", 0);
my $SX1DBPath    = Deps::MakePath($MXDExecDir, "SX-1.txt",     0);
if($OS eq 'MSWin32') {
}
elsif($OS eq 'linux') {
	$SX1DBPath = Deps::MakePath("$ENV{HOME}/bin/MD/MXD", "SX-1.txt", 0);
}
my $LDDir         = Deps::MakePath($ProgramDir, ["MD", "LD"],  0);

my $LAMMPSPerlDir = Deps::MakePath($TkPerlDir, ["LAMMPS"],  0);

my $GULPDir      = Deps::MakePath($ProgramDir, ["MD", "gulp-5.0"],  0);
my $GULPLibDir   = Deps::MakePath($GULPDir, "Libraries",     0);

my $WebRootDir      = "d:\\MyWebs";
$WebRootDir = "d:\\MyWebs" if(!-d $WebRootDir);
my $KListDBDir      = Deps::MakePath($WebRootDir, ["Research", "klist"], 1);
if($OS eq 'linux') {
   $KListDBDir      = Deps::MakePath($ENV{HOME}, ["bin", "klist"], 1);
   $KListDBDir      = Deps::MakePath("/home/share", ["bin", "klist"], 1) if(!-d $KListDBDir);
}

my $SGROUPPath = 'sgroup';
my $TEEPath = 'tee';
  if($OS eq 'MSWin32') {
	$SGROUPPath = Deps::MakePath("d:\\Programs\\Perl\\WIEN2k", "sgroup.exe");
	$SGROUPPath = Deps::MakePath("c:\\Programs\\Perl\\WIEN2k", "sgroup.exe") if(!-e $SGROUPPath);
	$TEEPath    = Deps::MakePath("d:\\Programs\\bin", "tee.exe");
	$TEEPath    = Deps::MakePath("c:\\Programs\\bin", "tee.exe") if(!-e $TEEPath);
  }

if($OS eq 'MSWin32') {
}
elsif($OS eq 'linux') {
}

sub OS                  { return $OS; }
sub BootDir {
	return Deps::ExtractDirectory($0);
}

sub Program             { return $Program; }
sub ProgramDir          { return $ProgramDir; }
sub ProgramOldDir       { return $ProgramOldDir; }
sub BaseDir             { return $BaseDir; }
sub TkPerlDir           {return $TkPerlDir; }

sub BinDir              { return $BinDir;  }
sub DBDir               { return $DBDir;   }
sub VASPDir             { return $VASPDir; }
sub VASPProgOldDir      { return $VASPProgOldDir; }
sub QEDir               { return $QEDir; };
sub QEPPDir             { return $QEPPDir; };
sub PHASEDir            { return $PHASEDir; }
sub PHASEPerlDir        { return $PHASEPerlDir; }
sub PHASEPotDir         { return $PHASEPotDir; }
sub CRYSTAL06Dir        { return $CRYSTAL06Dir; }
sub DVXaDir             { return $DVXaDir; }
sub SCIGRESSDir         { return $SCIGRESSDir; }
sub XRDDir              { return $XRDDir; }
sub RietanDir           { return $RietanDir; }
sub RietanTemplateDir   { return $RietanTemplateDir; }
sub RietanFPDir         { return $RietanFPDir; }
sub RietanFPTemplateDir { return $RietanFPTemplateDir; }
sub PeriodicTableDir    { return $PeriodicTableDir; }
sub MXDDir              { return $MXDDir; }
sub MXDExecDir          { return $MXDExecDir; }
sub MXDOrtoDir          { return $MXDOrtoDir; }
sub MXDInputPath        { return $MXDInputPath; }
sub SX1DBPath           { return $SX1DBPath; }
sub LDDir               { return $LDDir; }

sub LAMMPSPerlDir       { return $LAMMPSPerlDir; }
sub GULPDir             { return $GULPDir; }
sub GULPLibDir          { return $GULPLibDir; }

sub WebRootDir          { return $WebRootDir; }
sub KListDBDir          { return $KListDBDir; }

sub SGROUPPath          { return $SGROUPPath; }
sub TEEPath             { return $TEEPath; }

#============================================================
#　コンストラクタ、デストラクタ
#============================================================
sub new
{
	my ($module, $app) = @_;
	my $this = {};
	bless $this;
#	$this->SUPER::new(@_);
	return $this;
}

sub DESTROY
{
	my $this = shift;
	$this->SUPER::DESTROY(@_);
}
 
#============================================================
# 一般関数
#============================================================

sub ReadConfigure
{
	my ($this, $ini, $pParams, $UpdateVars, $PrintVars) = @_;
	$UpdateVars = 0 if(!defined $UpdateVars);
	$PrintVars = 0 if(!defined $PrintVars);
#print "0=$0\n";
#print "Program=$Program\n";

	my $BootDir = &BootDir();
	my $InFile = $ini;
	$InFile = Deps::MakePath($BootDir, $ini, 0) if(!-f $InFile);
	$InFile = Deps::MakePath($ProgramDir, $ini, 0) if(!-f $InFile);

	if($PrintVars) {
		print "ProgVars::ReadConfigure: Read variables from [$InFile]\n";
	}

	my %v;
	my $in = JFile->new($InFile, 'r');
	return $pParams if(!$in);
	while(1) {
		my $line = $in->ReadLine();
		last if(!defined $line);
		next if($line =~ /^\s*#/);
		
		my ($key, $var) = ($line =~ /^\s*(\S+)\s*=\s*(.*?)\s*$/);
		next if(!defined $key);

#print "$key: $var\n";
		$var = Template->new()->ReplaceByHash($var, \%v, '{', '}');
		$v{$key} = $var;
		if($PrintVars) {
			print "  $key=$v{$key}\n";
		}
		$var =~ s/\\/\//g;
		if($UpdateVars) {
			my $cmd = "\$$key = '$var'";
#print "cmd: $cmd\n";
			eval($cmd);
		}
	}
	$in->Close();
#foreach my $key (sort keys %v) {
#print "$key: $v{$key}\n";
#}
#exit;
	$pParams = Utils::MergeHash($pParams, \%v);

	if($PrintVars) {
		print "\n";
	}

	return $pParams;
}

"""
