import numpy as np
from numpy import sqrt
import physbo
import matplotlib.pyplot as plt
import pandas as pd

infile = 'data_simple.xlsx'
#infile = 'data_simple.csv'


figsize         = (8, 4)
figsize_plot    = (12, 8)
fontsize        = 16
legend_fontsize = 8


def func(x, y):
    return -1.0 * ((x - 0.2)**2 + (y - 0.3)**2)

def load_data():
    if '.xlsx' in infile:
        df =  pd.read_excel(infile, engine = 'openpyxl')
    else:
        df =  pd.read_csv(infile)

    columns = df.columns
    ncol  = len(columns)
    ndata = len(df.index)

    t_all = df[columns[0]].to_numpy()
    x_all = df[columns[1:]].to_numpy()

    df2 = df.dropna(how = 'any')
    i_train = df2.index.to_numpy()
#    df2 = df2.reset_index()
    t_train = df2[columns[0]].to_numpy()
    x_train = df2[columns[1:]].to_numpy()
    return i_train, x_train, t_train, x_all, t_all

i_train, X_train, t_train, X_all, t_all = load_data()
X = physbo.misc.centering(X_all)

t_exact = []
for i in range(len(t_all)):
    x, y = X_all[i]
    t_exact.append(func(x, y))

#N = len(t_train)

#共分散
num_dim = X_train.shape[1]
print("num_dim=", num_dim)
cov = physbo.gp.cov.gauss(num_dim, ard = False )
#平均
mean = physbo.gp.mean.const()
#尤度関数
lik = physbo.gp.lik.gauss()
#ガウス過程モデルの生成
gp = physbo.gp.model(lik = lik, mean = mean, cov = cov)
config = physbo.misc.set_config()
#ガウス過程モデルを学習
gp.fit(X_train, t_train, config)

#学習されたガウス過程におけるパラメタを出力
print("")
print("Learning results:")
gp.print_params()

#テストデータの平均値（予測値）および分散を計算
gp.prepare(X_train, t_train)
fmean = gp.get_post_fmean(X_train, X_all)
fcov  = gp.get_post_fcov(X_train, X_all)
fstd  = sqrt(fcov)
#予測の結果
print("fmean=", fmean)
#分散の結果
print("fcov=", fcov)
#予測値の平均二乗誤差の出力
#print("mean=", np.mean((fmean - t_test)**2))

#=====================================================
# plot
#=====================================================

fig = plt.figure(figsize = figsize_plot)
ax1 = fig.add_subplot(1, 1, 1)

x = range(len(fmean))
ax1.plot(i_train, t_train, label = 'train', marker = 'o', linestyle ='')
ax1.plot(x, t_exact, label = 'exact', linewidth = 0.5, color = 'red')
ax1.plot(x, fmean, label = 'mean', linewidth = 1.0, color = 'black')
#ax1.plot(x, fmean + fstd, linewidth = 0.5, color = 'green')
#ax1.plot(x, fmean - fstd, linewidth = 0.5, color = 'green')
ax1.fill_between(x, fmean - fstd, fmean + fstd, color='b', alpha = 0.1)
ax1.legend()


def find_nearest_data(x, y, xlist, ylist):
    minr2 = 1.0e300
    ihit = None
    for i in range(len(xlist)):
        xi = xlist[i]
        yi = ylist[i]
        r2 = (x - xi)**2 + (y - yi)**2
        if minr2 > r2:
            minr2 = r2
            ihit  = i
    return ihit, minr2

def onclick(event):
    if event.inaxes != ax1:
        return

    xe, ye  = event.xdata, event.ydata
    idx, r2 = find_nearest_data(xe, ye, x, fmean)
    print("")
    ndata = len(t_all)
    print(f"clicked at idx={idx}: x=", X_all[idx])
    print( "                       t=", t_all[idx])
    print(f"   predict={fmean[idx]} +- {fstd[idx]}")

fig.canvas.mpl_connect("button_press_event", onclick)
#fig.canvas.mpl_connect("motion_notify_event", hover)


plt.show()

