import sys
import numpy as np
import openpyxl
import pandas as pd
import matplotlib.pyplot as plt


norder = 3
infile  =  'random-poly.xlsx'

fontsize = 16


argv = sys.argv
narg = len(argv)
if narg >= 2:
    infile = argv[1]
if narg >= 3:
    norder = int(argv[2])


def main():
    print("Least-squares method for polynomial order {}".format(norder))
    print(f"norder={norder}")
    print(f"infile={infile}")

    print("")
    print(f"Read [{infile}]")
    df = pd.read_excel(infile, engine = 'openpyxl')
    labels = df.columns.to_list()
    x = df[labels[0]]
    y = df[labels[1]]
    ndata = len(x)

    print("")
    print(f"Execute linear least-squares method")
    ci = np.polyfit(x, y, norder)

    print("LSQ function")
    print("f(x) = {}".format(ci[norder]), end = '')
    for i in range(norder, 0, -1):
        print(" + {} * x^{}".format(ci[i], i), end = '')
    print("")


    ycal =  np.poly1d(ci)(x)

    """
# poly1dでこの部分の計算をしてくれている
    ycal = []
    for i in range(ndata):
        _x = x[i]
        yl = 0.0
        for k in range(norder+1):
            yl += ci[k] * pow(_x, norder - k)

        ycal.append(yl)
    """

#================================================================
# Plot
#================================================================
    fig, axes = plt.subplots(1, 2, figsize = (8, 6))
    axes[0].plot(x, y,    label = 'input', linestyle = '', marker = 'o')
    axes[0].plot(x, ycal, label = 'fit',   linestyle = '-')
    axes[0].tick_params(labelsize = fontsize)
    axes[0].set_xlabel('$x$', fontsize = fontsize)
    axes[0].set_ylabel('$y$', fontsize = fontsize)
    axes[0].legend(fontsize = fontsize)

    axes[1].plot(range(norder+1), ci, label = 'coeff', linestyle = '', marker = 'o')
    axes[1].tick_params(labelsize = fontsize)
    axes[1].set_xlabel('$i$', fontsize = fontsize)
    axes[1].set_ylabel('$c_i$', fontsize = fontsize)
    axes[1].legend(fontsize = fontsize)

    plt.tight_layout()
    plt.pause(0.1)

    print("")
    print("Press ENTER to terminate")
    input()


if __name__ == "__main__":
    main()
